@@ -23,9 +23,29 @@
 
 #include <glib.h>
 #include <gtk/gtk.h>
-#include <gpapa.h>
-#include "xmalloc.h"
-#include "logging.h"
+#include <gpgme.h>
+#include <stdlib.h>
+#include <errno.h>
+#include <string.h>
+#include "config.h"
+#include "gpadefs.h"
+#include "gpgmetools.h"
+#include "gtkhacks.h"
+#include "keytable.h"
+#include "options.h"
+
+/* This is a *very* development version */
+#define IS_DEVELOPMENT_VERSION
+
+/* For mkdir() */
+#ifdef G_OS_WIN32
+#include <direct.h>
+#include <io.h>
+#define mkdir(p,m) _mkdir(p)
+#ifndef S_ISDIR
+#define S_ISDIR(mode) ((mode)&_S_IFDIR)
+#endif
+#endif
 
 #include "i18n.h"   /* fixme: file should go into each source file */
 #include "options.h" /* ditto */
@@ -33,32 +53,21 @@
 
 extern GtkWidget *global_windowMain;
 extern GtkWidget *global_windowTip;
-extern GpapaAction global_lastCallbackResult;
 extern GList *global_defaultRecipients;
+extern GpgmeCtx ctx;
+extern GPAKeyTable *keytable;
+extern GpaOptions *gpa_options;
 
-extern GtkWidget *gpa_get_global_clist_file (void);
-extern void gpa_callback (GpapaAction action, gpointer actiondata,
-			  gpointer calldata);
-extern void sigs_append (gpointer data, gpointer userData);
-extern void gpa_selectRecipient (GtkWidget * clist, gint row, gint column,
-				 GdkEventButton * event, gpointer userData);
-extern void gpa_unselectRecipient (GtkWidget * clist, gint row, gint column,
-				   GdkEventButton * event, gpointer userData);
-extern void gpa_removeRecipients (gpointer param);
-extern void gpa_addRecipients (gpointer param);
-extern void gpa_recipientWindow_close (gpointer param);
-
-gboolean gpa_simplified_ui (void);
-void gpa_set_simplified_ui (gboolean value);
-
-gchar * gpa_default_key (void);
+gchar *gpa_default_key (void);
 void gpa_set_default_key (gchar * key);
 void gpa_update_default_key (void);
 
 void gpa_open_keyring_editor (void);
 void gpa_open_filemanager (void);
+void gpa_open_settings_dialog (void);
 GtkWidget * gpa_get_keyring_editor (void);
 GtkWidget * gpa_get_filenamager (void);
+GtkWidget * gpa_get_settings_dialog (void);
 
 typedef void (*GPADefaultKeyChanged) (gpointer user_data);
 
