@@ -23,14 +23,13 @@
  *	specific widgets
  */
 
+#include "gpa.h"
+#include <time.h>
 #include <config.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
-#include <gpapa.h>
-
 #include "gpapastrings.h"
-#include "gpa.h"
 #include "gtktools.h"
 
 
@@ -40,31 +39,43 @@
  */ 
 
 GtkWidget *
-gpa_key_info_new (GpapaKey * key, GtkWidget * window)
+gpa_key_info_new (GpgmeKey key)
 {
   GtkWidget * table;
   GtkWidget * label;
+  gchar *string, *uid;
+  int i;
 
   table = gtk_table_new (2, 2, FALSE);
   gtk_table_set_col_spacing (GTK_TABLE (table), 0, 10);
   gtk_table_set_row_spacing (GTK_TABLE (table), 0, 0);
 
-  /* User Name */
-  label = gtk_label_new (_("User Name:"));
-  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-
-  label = gtk_label_new (gpapa_key_get_name (key, gpa_callback, window));
+  /* One user ID on each line */
+  string = gpa_gpgme_key_get_userid (key, 0);
+  for (i = 1; (uid = gpa_gpgme_key_get_userid (key, i)) != NULL; i++)
+    {
+      gchar *tmp = string;
+      string = g_strconcat (string, "\n", uid, NULL);
+      g_free (tmp);
+    }
+  label = gtk_label_new (string);
+  g_free (string);
   gtk_table_attach (GTK_TABLE (table), label, 1, 2, 0, 1,
-		    GTK_FILL|GTK_EXPAND, 0, 0, 0);
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+		    GTK_FILL, 0, 0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
 
   /* User Name */
+  label = gtk_label_new (i == 1 ? _("User Name:") : _("User Names:") );
+  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 
+                    0, 0);
+  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.0);
+
+  /* Key ID */
   label = gtk_label_new (_("Key ID:"));
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
 
-  label = gtk_label_new (gpapa_key_get_identifier (key, gpa_callback, window));
+  label = gtk_label_new (gpa_gpgme_key_get_short_keyid (key, 0));
   gtk_table_attach (GTK_TABLE (table), label, 1, 2, 1, 2,
 		    GTK_FILL|GTK_EXPAND, 0, 0, 0);
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
@@ -73,82 +84,112 @@ gpa_key_info_new (GpapaKey * key, GtkWidget * window)
 }
 
 /*
- *	A CList for choosing from secret keys
+ *	A set of CLists for choosing from a list of keys.
  */
 
-GtkWidget *
-gpa_secret_key_list_new (GtkWidget *window)
+/* Build the list */
+static GtkWidget *
+gpa_key_list_new (void)
 {
   GtkWidget *clist;
-  gint num_keys;
-  GpapaSecretKey *key;
-  gint default_key_index = 0;
-  gchar *titles[2] = {_("User Identity/Role"), _("Key ID")};
-  gchar *contents[2];
-  gint row;
+  gchar *titles[2] = {_("Key ID"), _("User Name")};
 
   clist = gtk_clist_new_with_titles (2, titles);
-
-  /* FIXME: widths shouldn't be hard-coded: */
-  gtk_clist_set_column_width (GTK_CLIST (clist), 0, 185);
-  gtk_clist_set_column_width (GTK_CLIST (clist), 1, 120);
-  num_keys = gpapa_get_secret_key_count (gpa_callback, window);
-  while (num_keys)
-    {
-      num_keys--;
-      key = gpapa_get_secret_key_by_index (num_keys, gpa_callback, window);
-      contents[0] = gpapa_key_get_name (GPAPA_KEY (key), gpa_callback, window);
-      contents[1] = gpapa_key_get_identifier (GPAPA_KEY (key), gpa_callback,
-					      window);
-      row = gtk_clist_prepend (GTK_CLIST (clist), contents);
-      gtk_clist_set_row_data_full (GTK_CLIST (clist), row,
-				   contents[1]? xstrdup (contents[1]):NULL,
-                                   free);
-      if (gpa_default_key () && strcmp (gpa_default_key (), contents[1]) == 0)
-	{
-	  default_key_index = num_keys;
-	}
-    } /* while */
+  gtk_clist_set_sort_column (GTK_CLIST (clist), 1);
+  gtk_clist_set_auto_sort (GTK_CLIST (clist), TRUE);
   gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_SINGLE);
   gtk_clist_column_title_passive (GTK_CLIST (clist), 0);
   gtk_clist_column_title_passive (GTK_CLIST (clist), 1);
-  gtk_clist_select_row (GTK_CLIST (clist), default_key_index, 0);
+  gtk_clist_set_column_auto_resize (GTK_CLIST (clist), 0, TRUE);
+  gtk_clist_set_column_auto_resize (GTK_CLIST (clist), 0, TRUE);
 
   return clist;
 }
 
-GtkWidget *
-gpa_public_key_list_new (GtkWidget *window)
+struct add_key_data_s
 {
   GtkWidget *clist;
-  gint num_keys;
-  GpapaPublicKey *key;
-  gchar *titles[2] = {_("User Identity/Role"), _("Key ID")};
+}; 
+
+static void
+gpa_key_list_add_key  (const gchar * fpr, GpgmeKey key, gpointer data)
+{
+  struct add_key_data_s *add_data = data;
   gchar *contents[2];
   gint row;
+  
+  contents[0] = g_strdup (gpa_gpgme_key_get_short_keyid (key, 0));
+  contents[1] = gpa_gpgme_key_get_userid (key, 0);
+  row = gtk_clist_prepend (GTK_CLIST (add_data->clist), contents);
+  gtk_clist_set_row_data_full (GTK_CLIST (add_data->clist), row,
+			       fpr ? g_strdup (fpr) : NULL, g_free);
+  g_free (contents[0]);
+  g_free (contents[1]);
+}
 
-  clist = gtk_clist_new_with_titles (2, titles);
+/* A list of all the secret keys, with the default key selected by default */
 
-  /* FIXME: widths shouldn't be hard-coded: */
-  gtk_clist_set_column_width (GTK_CLIST (clist), 0, 185);
-  gtk_clist_set_column_width (GTK_CLIST (clist), 1, 120);
-  num_keys = gpapa_get_public_key_count (gpa_callback, window);
-  while (num_keys)
+GtkWidget *
+gpa_secret_key_list_new (void)
+{
+  struct add_key_data_s data;
+  
+  data.clist = gpa_key_list_new ();
+  gtk_clist_freeze (GTK_CLIST (data.clist));
+  gpa_keytable_secret_foreach (keytable, gpa_key_list_add_key, &data);
+  gtk_clist_thaw (GTK_CLIST (data.clist));
+  
+  /* Select the default key */
+  if (gpa_options_get_default_key (gpa_options))
     {
-      num_keys--;
-      key = gpapa_get_public_key_by_index (num_keys, gpa_callback, window);
-      contents[0] = gpapa_key_get_name (GPAPA_KEY (key), gpa_callback, window);
-      contents[1] = gpapa_key_get_identifier (GPAPA_KEY (key), gpa_callback,
-					      window);
-      row = gtk_clist_prepend (GTK_CLIST (clist), contents);
-      gtk_clist_set_row_data_full (GTK_CLIST (clist), row,
-				   contents[1]?xstrdup (contents[1]):NULL,
-                                   free);
-    } /* while */
-  gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_SINGLE);
-  gtk_clist_column_title_passive (GTK_CLIST (clist), 0);
-  gtk_clist_column_title_passive (GTK_CLIST (clist), 1);
+      gint i;
+      gchar *fpr;
+      
+      for (i = 0; (fpr = gtk_clist_get_row_data (GTK_CLIST (data.clist), i));
+           i++)
+        {
+          if (g_str_equal (gpa_options_get_default_key (gpa_options), fpr))
+            {
+              gtk_clist_select_row (GTK_CLIST (data.clist), i, 0);
+            }
+        }
+    }
 
+  return data.clist;
+}
+
+/* A list of all the public keys */
+
+GtkWidget *
+gpa_public_key_list_new (void)
+{
+  struct add_key_data_s data;
+  
+  data.clist = gpa_key_list_new ();
+  gtk_clist_freeze (GTK_CLIST (data.clist));
+  gpa_keytable_foreach (keytable, gpa_key_list_add_key, &data);
+  gtk_clist_thaw (GTK_CLIST (data.clist));
+
+  return data.clist;
+}
+
+/* A list of all the keys in a GList */
+
+GtkWidget *
+gpa_key_list_new_from_glist (GtkWidget *window, GList *list)
+{
+  GtkWidget *clist = NULL;
+  /* FIXME: Reimplement when it's needed (disabled until we know how to list
+   * keys from a remote server) */
+#if 0
+  gint default_key_index = 0;
+
+  clist = gpa_key_list_new (window, 
+                            g_list_length (list),
+                            (GpapaKey* (*) (gint, GpapaCallbackFunc,
+                                            gpointer, void*)) get_clist_key, 
+                            &default_key_index, list);
+#endif
   return clist;
 }
 
@@ -211,7 +252,7 @@ gpa_expiry_frame_free (gpointer param)
   GPAExpiryFrame * frame = param;
   if (frame->expiryDate)
     g_date_free (frame->expiryDate);
-  free (frame);
+  g_free (frame);
 }
 
 static void
@@ -258,11 +299,13 @@ gpa_expiry_frame_at (GtkToggleButton * radioAt, gpointer param)
   gtk_entry_set_text (GTK_ENTRY (frame->entryAfter), "");
   if (frame->expiryDate)
     {
-      dateBuffer = gpa_expiry_date_string (frame->expiryDate);
+      struct tm tm;
+      g_date_to_struct_tm (frame->expiryDate, &tm);
+      dateBuffer = gpa_expiry_date_string (mktime (&tm));
       gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (frame->comboAfter)->entry),
 			  "days");
       gtk_entry_set_text (GTK_ENTRY (frame->entryAt), dateBuffer);
-      free (dateBuffer);
+      g_free (dateBuffer);
     } /* if */
   else
     gtk_entry_set_text (GTK_ENTRY (frame->entryAt), _("01.01.2000")); /*!!! */
@@ -289,7 +332,7 @@ gpa_expiry_frame_new (GtkAccelGroup * accelGroup, GDate * expiryDate)
 
   GPAExpiryFrame * frame;
 
-  frame = xmalloc(sizeof(*frame));
+  frame = g_malloc(sizeof(*frame));
   frame->expiryDate = expiryDate;
 
   expiry_frame = gtk_frame_new (_("Expiration"));
@@ -314,8 +357,7 @@ gpa_expiry_frame_new (GtkAccelGroup * accelGroup, GDate * expiryDate)
   entryAfter = gtk_entry_new ();
   frame->entryAfter = entryAfter;
   gtk_widget_set_usize (entryAfter,
-			gdk_string_width (entryAfter->style->font, " 00000 "),
-			0);
+    gdk_string_width (gtk_style_get_font (entryAfter->style), " 00000 "), 0);
   gtk_box_pack_start (GTK_BOX (hboxAfter), entryAfter, FALSE, FALSE, 0);
 
   comboAfter = gtk_combo_new ();
@@ -338,9 +380,11 @@ gpa_expiry_frame_new (GtkAccelGroup * accelGroup, GDate * expiryDate)
   frame->entryAt = entryAt;
   if (expiryDate)
     {
-      dateBuffer = gpa_expiry_date_string (expiryDate);
+      struct tm tm;
+      g_date_to_struct_tm (expiryDate, &tm);
+      dateBuffer = gpa_expiry_date_string (mktime (&tm));
       gtk_entry_set_text (GTK_ENTRY (entryAt), dateBuffer);
-      free (dateBuffer);
+      g_free (dateBuffer);
     } /* if */
   gtk_box_pack_start (GTK_BOX (hboxAt), entryAt, FALSE, FALSE, 0);
 
@@ -369,7 +413,7 @@ gpa_expiry_frame_get_expiration(GtkWidget * expiry_frame, GDate ** date,
 {
   GPAExpiryFrame * frame = gtk_object_get_data (GTK_OBJECT (expiry_frame),
 						"user_data");
-  gchar * temp;
+  gchar *temp;
   gboolean result = FALSE;
 
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (frame->radioDont)))
@@ -381,7 +425,7 @@ gpa_expiry_frame_get_expiration(GtkWidget * expiry_frame, GDate ** date,
   else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(frame->radioAfter)))
     {
       *interval = atoi (gtk_entry_get_text (GTK_ENTRY(frame->entryAfter)));
-      temp =gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(frame->comboAfter)->entry));
+      temp = (gchar *) gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(frame->comboAfter)->entry));
       *unit = gpa_time_unit_from_string (temp);
       *date = NULL;
       result = TRUE;
@@ -436,7 +480,7 @@ gpa_expiry_frame_validate(GtkWidget * expiry_frame)
 			gtk_entry_get_text (GTK_ENTRY (frame->entryAt)));
       if (!g_date_valid (date))
 	{
-	  result = _("Please provide a correct date");
+	  result = _("Please provide a correct date.");
 	}
       else
 	{
