@@ -18,12 +18,14 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include "gpa.h"
 #include <config.h>
 #include <stdlib.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
-#include "gpa.h"
+#include "gtktools.h"
 #include "gpawindowkeeper.h"
+#include "gpafilesel.h"
 #include "icons.h"
 
 
@@ -283,6 +285,7 @@ gpa_button_set_text (GtkWidget * button, gchar * text,
 
   accel = gtk_label_parse_uline (GTK_LABEL (label), text);
 
+#if 0  /* @@@@@@ */
   /* In GTK 1.2.8, the visible_only parameter of
    * gtk_widget_remove_accelerators is not even looked at, it always
    * behaves as if it were TRUE. Therefore make sure that
@@ -295,233 +298,76 @@ gpa_button_set_text (GtkWidget * button, gchar * text,
       gtk_widget_add_accelerator (button, "clicked", accel_group,
 				  accel, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
     }
+#endif
 }
 
 void
-gpa_window_error (gchar * message, GtkWidget * messenger)
+gpa_window_error (const gchar *message, GtkWidget *messenger)
 {
-  GpaWindowKeeper *keeper;
-  GtkAccelGroup *accelGroup;
   GtkWidget *windowError;
-  GtkWidget *vboxError;
+  GtkWidget *hboxError;
   GtkWidget *labelMessage;
-  GtkWidget *buttonClose;
+  GtkWidget *pixmap;
 
-  keeper = gpa_windowKeeper_new ();
-  windowError = gtk_window_new (GTK_WINDOW_DIALOG);
-  gpa_windowKeeper_set_window (keeper, windowError);
-  gtk_window_set_title (GTK_WINDOW (windowError), _("GPA Error"));
-  gtk_window_set_modal (GTK_WINDOW (windowError), TRUE);
-  accelGroup = gtk_accel_group_new ();
-  gtk_window_add_accel_group (GTK_WINDOW (windowError), accelGroup);
-  vboxError = gtk_vbox_new (FALSE, 0);
-  gtk_container_set_border_width (GTK_CONTAINER (vboxError), 10);
+  windowError = gtk_dialog_new_with_buttons (_("GPA Error"),
+                                             (messenger ? 
+                                             GTK_WINDOW(messenger) : NULL),
+                                             GTK_DIALOG_MODAL,
+                                             GTK_STOCK_CLOSE,
+                                             GTK_RESPONSE_CLOSE,
+                                             NULL);
+  gtk_container_set_border_width (GTK_CONTAINER (windowError), 5);
+  gtk_dialog_set_default_response (GTK_DIALOG (windowError),
+                                   GTK_RESPONSE_CLOSE);
+  hboxError = gtk_hbox_new (FALSE, 0);
+  gtk_container_set_border_width (GTK_CONTAINER (hboxError), 5);
+  gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (windowError)->vbox),
+                               hboxError);
+  pixmap = gtk_image_new_from_stock (GTK_STOCK_DIALOG_ERROR,
+                                     GTK_ICON_SIZE_DIALOG);
+  gtk_box_pack_start (GTK_BOX (hboxError), pixmap, TRUE, FALSE, 10);
   labelMessage = gtk_label_new (message);
-  gtk_box_pack_start (GTK_BOX (vboxError), labelMessage, TRUE, FALSE, 10);
-  buttonClose = gpa_button_new (accelGroup, _("_OK"));
-  gtk_signal_connect_object (GTK_OBJECT (buttonClose), "clicked",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     (gpointer) windowError);
-  gtk_widget_add_accelerator (buttonClose, "clicked", accelGroup, GDK_Escape,
-			      0, 0);
-  gtk_box_pack_start (GTK_BOX (vboxError), buttonClose, FALSE, FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (windowError), vboxError);
+  gtk_box_pack_start (GTK_BOX (hboxError), labelMessage, TRUE, FALSE, 10);
 
-  if (messenger)
-    gpa_window_show_centered (windowError, messenger);
-  else
-    gtk_widget_show_all (windowError);
-  gtk_widget_grab_focus (buttonClose);
+  gtk_widget_show_all (windowError);
+  gtk_dialog_run (GTK_DIALOG (windowError));
+  gtk_widget_destroy (windowError);
 } /* gpa_window_error */
 
 void
 gpa_window_message (gchar * message, GtkWidget * messenger)
 {
-/* var */
-  GpaWindowKeeper *keeper;
-  GtkAccelGroup *accelGroup;
-/* objects */
-  GtkWidget *windowMessage;
-  GtkWidget *vboxMessage;
+  GtkWidget *window;
+  GtkWidget *hbox;
   GtkWidget *labelMessage;
-  GtkWidget *buttonClose;
-/* commands */
-  keeper = gpa_windowKeeper_new ();
-  windowMessage = gtk_window_new (GTK_WINDOW_DIALOG);
-  gpa_windowKeeper_set_window (keeper, windowMessage);
-  gtk_window_set_title (GTK_WINDOW (windowMessage), _("GPA Message"));
-  gtk_window_set_modal (GTK_WINDOW (windowMessage), TRUE);
-  accelGroup = gtk_accel_group_new ();
-  gtk_window_add_accel_group (GTK_WINDOW (windowMessage), accelGroup);
-  vboxMessage = gtk_vbox_new (FALSE, 0);
-  gtk_container_set_border_width (GTK_CONTAINER (vboxMessage), 10);
+  GtkWidget *pixmap;
+
+  window = gtk_dialog_new_with_buttons (_("GPA Message"),
+					(messenger ? 
+					 GTK_WINDOW(messenger) : NULL),
+					GTK_DIALOG_MODAL,
+					GTK_STOCK_CLOSE,
+					GTK_RESPONSE_CLOSE,
+					NULL);
+  gtk_container_set_border_width (GTK_CONTAINER (window), 5);
+  gtk_dialog_set_default_response (GTK_DIALOG (window),
+                                   GTK_RESPONSE_CLOSE);
+  hbox = gtk_hbox_new (FALSE, 0);
+  gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
+  gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (window)->vbox),
+                               hbox);
+  pixmap = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO,
+                                     GTK_ICON_SIZE_DIALOG);
+  gtk_box_pack_start (GTK_BOX (hbox), pixmap, TRUE, FALSE, 10);
   labelMessage = gtk_label_new (message);
-  gtk_box_pack_start (GTK_BOX (vboxMessage), labelMessage, TRUE, FALSE, 10);
-  buttonClose = gpa_button_new (accelGroup, _("_OK"));
-  gtk_signal_connect_object (GTK_OBJECT (buttonClose), "clicked",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     (gpointer) windowMessage);
-  gtk_widget_add_accelerator (buttonClose, "clicked", accelGroup, GDK_Escape,
-			      0, 0);
-  gtk_box_pack_start (GTK_BOX (vboxMessage), buttonClose, FALSE, FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (windowMessage), vboxMessage);
-  gpa_window_show_centered (windowMessage, messenger);
-  gtk_widget_grab_focus (buttonClose);
+  gtk_box_pack_start (GTK_BOX (hbox), labelMessage, TRUE, FALSE, 10);
+
+  gtk_widget_show_all (window);
+  gtk_dialog_run (GTK_DIALOG (window));
+  gtk_widget_destroy (window);
 }				/* gpa_window_message */
 
 
-/* a simple but flexible message box */
-
-
-typedef struct {
-  gchar * result;
-  GtkWidget * window;
-} GPAMessageBox;
-
-static void
-message_box_destroy (GtkWidget *widget, gpointer param)
-{
-  gtk_main_quit ();
-}
-
-static void
-message_box_clicked (GtkWidget * button, gpointer param)
-{
-  GPAMessageBox * dialog = param;
-
-  dialog->result = gtk_object_get_data (GTK_OBJECT (button), "user_data");
-  gtk_widget_destroy (dialog->window);
-}
-
-gchar *
-gpa_message_box_run (GtkWidget * parent, const gchar * title,
-		     const gchar * message,
-		     const gchar ** buttons)
-{
-  GtkAccelGroup *accel_group;
-
-  GtkWidget * window;
-  GtkWidget * vbox;
-  GtkWidget * label;
-  GtkWidget * bbox;
-  GtkWidget * button;
-  int i;
-  GPAMessageBox dialog;
-  
-
-  dialog.result = NULL;
-
-  window = gtk_window_new (GTK_WINDOW_DIALOG);
-  dialog.window = window;
-  gtk_window_set_title (GTK_WINDOW (window), title);
-  gtk_signal_connect (GTK_OBJECT (window), "destroy",
-		      GTK_SIGNAL_FUNC (message_box_destroy),
-		      NULL);
-
-  accel_group = gtk_accel_group_new ();
-  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
-
-  vbox = gtk_vbox_new (FALSE, 5);
-  gtk_container_add (GTK_CONTAINER (window), vbox);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
-
-  label = gtk_label_new (message);
-  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
-  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-
-  bbox = gtk_hbutton_box_new ();
-  gtk_box_pack_start (GTK_BOX (vbox), bbox, TRUE, TRUE, 0);
-  gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_SPREAD);
-  gtk_button_box_set_spacing (GTK_BUTTON_BOX (bbox), 10);
-
-  for (i = 0; buttons[i]; i++)
-    {
-      button = gpa_button_new (accel_group, (gchar*)(buttons[i]));
-      gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
-      gtk_object_set_data (GTK_OBJECT (button), "user_data",
-			   (gpointer)buttons[i]);
-      gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			  GTK_SIGNAL_FUNC (message_box_clicked),
-			  (gpointer)&dialog);
-    }
-
-  gtk_window_set_modal (GTK_WINDOW (window), TRUE);
-  gpa_window_show_centered (window, parent);
-
-  gtk_main ();
-
-  return dialog.result;
-}
-  
-
-void
-gpa_window_passphrase (GtkWidget * messenger, GtkSignalFunc func, gchar * tip,
-		       gpointer data)
-{
-/* var */
-  GpaWindowKeeper *keeper;
-  GtkAccelGroup *accelGroup;
-  gpointer *param;
-  gpointer *paramClose;
-/* objects */
-  GtkWidget *windowPassphrase;
-  GtkWidget *vboxPassphrase;
-  GtkWidget *hboxPasswd;
-  GtkWidget *labelPasswd;
-  GtkWidget *entryPasswd;
-  GtkWidget *hButtonBoxPassphrase;
-  GtkWidget *buttonCancel;
-  GtkWidget *buttonOK;
-/* commands */
-  keeper = gpa_windowKeeper_new ();
-  windowPassphrase = gtk_window_new (GTK_WINDOW_DIALOG);
-  gpa_windowKeeper_set_window (keeper, windowPassphrase);
-  gpa_windowKeeper_add_param (keeper, data);
-  gtk_window_set_title (GTK_WINDOW (windowPassphrase), _("Insert password"));
-  accelGroup = gtk_accel_group_new ();
-  gtk_window_add_accel_group (GTK_WINDOW (windowPassphrase), accelGroup);
-  vboxPassphrase = gtk_vbox_new (FALSE, 0);
-  gtk_container_set_border_width (GTK_CONTAINER (vboxPassphrase), 5);
-  hboxPasswd = gtk_hbox_new (FALSE, 0);
-  gtk_container_set_border_width (GTK_CONTAINER (hboxPasswd), 5);
-  labelPasswd = gtk_label_new ("");
-  gtk_box_pack_start (GTK_BOX (hboxPasswd), labelPasswd, FALSE, FALSE, 0);
-  entryPasswd = gtk_entry_new ();
-  gtk_entry_set_visibility (GTK_ENTRY (entryPasswd), FALSE);
-  param = (gpointer *) xmalloc (3 * sizeof (gpointer));
-  gpa_windowKeeper_add_param (keeper, param);
-  param[0] = data;
-  param[1] = entryPasswd;
-  param[2] = keeper;
-  gtk_signal_connect_object (GTK_OBJECT (entryPasswd), "activate",
-			     GTK_SIGNAL_FUNC (func), (gpointer) param);
-  gtk_box_pack_start (GTK_BOX (hboxPasswd), entryPasswd, TRUE, TRUE, 0);
-  gpa_connect_by_accelerator (GTK_LABEL (labelPasswd), entryPasswd,
-			      accelGroup, _("_Password: "));
-  gtk_box_pack_start (GTK_BOX (vboxPassphrase), hboxPasswd, TRUE, TRUE, 0);
-  hButtonBoxPassphrase = gtk_hbutton_box_new ();
-  gtk_button_box_set_layout (GTK_BUTTON_BOX (hButtonBoxPassphrase),
-			     GTK_BUTTONBOX_END);
-  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hButtonBoxPassphrase), 10);
-  paramClose = (gpointer *) xmalloc (2 * sizeof (gpointer));
-  gpa_windowKeeper_add_param (keeper, paramClose);
-  paramClose[0] = keeper;
-  paramClose[1] = tip;
-  buttonCancel = gpa_buttonCancel_new (accelGroup, _("_Cancel"), paramClose);
-  gtk_container_set_border_width (GTK_CONTAINER (hButtonBoxPassphrase), 5);
-  gtk_container_add (GTK_CONTAINER (hButtonBoxPassphrase), buttonCancel);
-  buttonOK = gpa_button_new (accelGroup, _("_OK"));
-  gtk_signal_connect_object (GTK_OBJECT (buttonOK), "clicked",
-			     GTK_SIGNAL_FUNC (func), (gpointer) param);
-  gtk_container_add (GTK_CONTAINER (hButtonBoxPassphrase), buttonOK);
-  gtk_box_pack_start (GTK_BOX (vboxPassphrase), hButtonBoxPassphrase, FALSE,
-		      FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (windowPassphrase), vboxPassphrase);
-  gpa_window_show_centered (windowPassphrase, messenger);
-  gtk_widget_grab_focus (entryPasswd);
-}				/* gpa_window_passphrase */
-
 /*
  * Modal file dialog
  */
@@ -535,11 +381,12 @@ typedef struct _GPASaveFileNameDialog GPASaveFileNameDialog;
 static void
 file_dialog_ok (gpointer param)
 {
-  GPASaveFileNameDialog * dialog = param;
+  GPASaveFileNameDialog *dialog = param;
 
   dialog->filename 
-      = gtk_file_selection_get_filename (GTK_FILE_SELECTION (dialog->window));
-  dialog->filename = xstrdup_or_null (dialog->filename);
+      = (gchar *) gpa_file_selection_get_filename (GPA_FILE_SELECTION (dialog->window));
+  if( dialog->filename != NULL )
+	  dialog->filename = g_strdup (dialog->filename);
 
   gtk_widget_destroy (dialog->window);
 }
@@ -566,7 +413,7 @@ gpa_get_save_file_name (GtkWidget * parent, const gchar * title,
 			const gchar * directory)
 {
   GPASaveFileNameDialog dialog;
-  GtkWidget * window = gtk_file_selection_new (title);
+  GtkWidget * window = gpa_file_selection_new (title);
 
   dialog.window = window;
   dialog.filename = NULL;
@@ -574,11 +421,11 @@ gpa_get_save_file_name (GtkWidget * parent, const gchar * title,
   gtk_signal_connect (GTK_OBJECT (window), "destroy",
 		      GTK_SIGNAL_FUNC (file_dialog_destroy), NULL);
 
-  gtk_signal_connect_object (GTK_OBJECT(GTK_FILE_SELECTION(window)->ok_button),
+  gtk_signal_connect_object (GTK_OBJECT(GPA_FILE_SELECTION(window)->ok_button),
 			     "clicked", GTK_SIGNAL_FUNC (file_dialog_ok),
 			     (gpointer) &dialog);
   gtk_signal_connect_object (
-		      GTK_OBJECT (GTK_FILE_SELECTION (window)->cancel_button),
+		      GTK_OBJECT (GPA_FILE_SELECTION (window)->cancel_button),
 		      "clicked", GTK_SIGNAL_FUNC (file_dialog_cancel),
 		      (gpointer) &dialog);
 
