@@ -22,7 +22,7 @@
 #define KEYDELETEDLG_H
 
 #include <gtk/gtk.h>
-gboolean gpa_delete_dialog_run (GtkWidget * parent, GpapaPublicKey * key,
+gboolean gpa_delete_dialog_run (GtkWidget * parent, GpgmeKey key,
 				gboolean has_secret_key);
 
 #endif /* KEYDELETEDLG_H */
