@@ -25,6 +25,8 @@
 #include "gtktools.h"
 #include "keygendlg.h"
 
+#define	XSTRDUP_OR_NULL(s)	((s != NULL) ? g_strdup(s): NULL)
+
 struct _GPAKeyGenDialog {
   GtkWidget * window;
   GtkWidget * entryPasswd;
@@ -35,33 +37,6 @@ struct _GPAKeyGenDialog {
 typedef struct _GPAKeyGenDialog GPAKeyGenDialog;
 
 
-static GPAKeyGenParameters *
-key_gen_params_new(void)
-{
-  GPAKeyGenParameters * params = xmalloc (sizeof (*params));
-  params->userID = NULL;
-  params->email = NULL;
-  params->comment = NULL;
-  params->expiryDate = NULL;
-  params->interval = 0;
-  params->generate_revocation = FALSE;
-  params->send_to_server = FALSE;
-  return params;
-}
-
-void
-gpa_key_gen_free_parameters(GPAKeyGenParameters * params)
-{
-  free (params->userID);
-  free (params->email);
-  free (params->comment);
-  if (params->expiryDate)
-    g_date_free (params->expiryDate);
-  free (params);
-}
-
-
-
 static void
 key_gen_cancel (gpointer param)
 {
@@ -80,8 +55,7 @@ key_gen_ok (gpointer param)
   if (strcmp (gtk_entry_get_text (GTK_ENTRY (dialog->entryPasswd)),
 	      gtk_entry_get_text (GTK_ENTRY (dialog->entryRepeat))) != 0)
     {
-      gpa_window_error (_("In \"Password\" and \"Repeat Password\""
-			  "\nyou must enter the same password."),
+      gpa_window_error (_("In \"Passphrase\" and \"Repeat passphrase\"\nyou must enter the same passphrase."),
 			dialog->window);
     }
   else if ((expiry_error = gpa_expiry_frame_validate (dialog->frameExpire)))
@@ -136,14 +110,14 @@ gpa_key_gen_run_dialog (GtkWidget * parent)
   GPAKeyGenParameters * params = NULL;
 
   GList *contentsAlgorithm = NULL;
-  GpapaAlgo algo;
+  GPAKeyGenAlgo algo;
   GList *contentsKeysize = NULL;
 
   dialog.result = FALSE;
 
   accelGroup = gtk_accel_group_new ();
 
-  windowGenerate = gtk_window_new (GTK_WINDOW_DIALOG);
+  windowGenerate = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   dialog.window = windowGenerate;
   gtk_window_set_title (GTK_WINDOW (windowGenerate), _("Generate key"));
   gtk_window_add_accel_group (GTK_WINDOW (windowGenerate), accelGroup);
@@ -167,9 +141,9 @@ gpa_key_gen_run_dialog (GtkWidget * parent)
   comboAlgorithm = gtk_combo_new ();
   gtk_editable_set_editable (GTK_EDITABLE (GTK_COMBO (comboAlgorithm)->entry),
 			     FALSE);
-  for (algo = GPAPA_ALGO_FIRST; algo <= GPAPA_ALGO_LAST; algo++)
+  for (algo = GPA_KEYGEN_ALGO_FIRST; algo <= GPA_KEYGEN_ALGO_LAST; algo++)
     contentsAlgorithm = g_list_append (contentsAlgorithm,
-				       gpa_algorithm_string(algo));
+				       (gchar*) gpa_algorithm_string(algo));
   gtk_combo_set_popdown_strings (GTK_COMBO (comboAlgorithm),
 				 contentsAlgorithm);
   gpa_connect_by_accelerator (GTK_LABEL (labelAlgorithm),
@@ -223,7 +197,7 @@ gpa_key_gen_run_dialog (GtkWidget * parent)
 		    GTK_FILL, GTK_SHRINK, 0, 0);
   entryEmail = gtk_entry_new ();
   gpa_connect_by_accelerator (GTK_LABEL (labelEmail), entryEmail, accelGroup,
-			      _("E-_Mail: "));
+			      _("_Email: "));
   gtk_table_attach (GTK_TABLE (tableMisc), entryEmail, 1, 2, 1, 2, GTK_FILL,
 		    GTK_SHRINK, 0, 0);
 
@@ -233,7 +207,7 @@ gpa_key_gen_run_dialog (GtkWidget * parent)
 		    GTK_FILL, GTK_SHRINK, 0, 0);
   entryComment = gtk_entry_new ();
   gpa_connect_by_accelerator (GTK_LABEL (labelComment), entryComment,
-			      accelGroup, _("C_omment: "));
+			      accelGroup, _("_Comment: "));
   gtk_table_attach (GTK_TABLE (tableMisc), entryComment, 1, 2, 2, 3, GTK_FILL,
 		    GTK_SHRINK, 0, 0);
 
@@ -245,7 +219,7 @@ gpa_key_gen_run_dialog (GtkWidget * parent)
   dialog.entryPasswd = entryPasswd;
   gtk_entry_set_visibility (GTK_ENTRY (entryPasswd), FALSE);
   gpa_connect_by_accelerator (GTK_LABEL (labelPasswd), entryPasswd,
-			      accelGroup, _("_Password: "));
+			      accelGroup, _("_Passphrase: "));
   gtk_table_attach (GTK_TABLE (tableMisc), entryPasswd, 1, 2, 3, 4, GTK_FILL,
 		    GTK_SHRINK, 0, 0);
 
@@ -257,7 +231,7 @@ gpa_key_gen_run_dialog (GtkWidget * parent)
   dialog.entryRepeat = entryRepeat;
   gtk_entry_set_visibility (GTK_ENTRY (entryRepeat), FALSE);
   gpa_connect_by_accelerator (GTK_LABEL (labelRepeat), entryRepeat,
-			      accelGroup, _("_Repeat password: "));
+			      accelGroup, _("_Repeat passphrase: "));
   gtk_table_attach (GTK_TABLE (tableMisc), entryRepeat, 1, 2, 4, 5,
 		    GTK_FILL, GTK_SHRINK, 0, 0);
 
@@ -280,11 +254,6 @@ gpa_key_gen_run_dialog (GtkWidget * parent)
   gtk_button_box_set_spacing (GTK_BUTTON_BOX (hButtonBoxGenerate), 10);
   gtk_container_set_border_width (GTK_CONTAINER (hButtonBoxGenerate), 5);
 
-  buttonCancel = gpa_button_cancel_new(accelGroup, _("_Cancel"),
-				       GTK_SIGNAL_FUNC (key_gen_cancel),
-				       (gpointer)&dialog);
-  gtk_container_add (GTK_CONTAINER (hButtonBoxGenerate), buttonCancel);
-
   buttonGenerate = gpa_button_new (accelGroup, _("_Generate key"));
 
   gtk_signal_connect_object (GTK_OBJECT (buttonGenerate), "clicked",
@@ -292,6 +261,11 @@ gpa_key_gen_run_dialog (GtkWidget * parent)
 			     (gpointer) &dialog);
   gtk_container_add (GTK_CONTAINER (hButtonBoxGenerate), buttonGenerate);
 
+  buttonCancel = gpa_button_cancel_new(accelGroup, _("_Cancel"),
+				       GTK_SIGNAL_FUNC (key_gen_cancel),
+				       (gpointer)&dialog);
+  gtk_container_add (GTK_CONTAINER (hButtonBoxGenerate), buttonCancel);
+
   gpa_window_show_centered (windowGenerate, parent);
 
   gtk_grab_add (windowGenerate);
@@ -306,15 +280,15 @@ gpa_key_gen_run_dialog (GtkWidget * parent)
       gchar * temp;
 
       params = key_gen_params_new ();
-      params->userID = xstrdup_or_null (gtk_entry_get_text (GTK_ENTRY (entryUserID)));
-      params->email = xstrdup_or_null(gtk_entry_get_text (GTK_ENTRY (entryEmail)));
-      params->comment = xstrdup_or_null (gtk_entry_get_text (GTK_ENTRY(entryComment)));
+      params->userID = XSTRDUP_OR_NULL (gtk_entry_get_text (GTK_ENTRY (entryUserID)));
+      params->email = XSTRDUP_OR_NULL(gtk_entry_get_text (GTK_ENTRY (entryEmail)));
+      params->comment = XSTRDUP_OR_NULL (gtk_entry_get_text (GTK_ENTRY(entryComment)));
 
-      params->password = xstrdup_or_null (gtk_entry_get_text (GTK_ENTRY(entryPasswd)));
+      params->password = XSTRDUP_OR_NULL (gtk_entry_get_text (GTK_ENTRY(entryPasswd)));
 	  
-      temp = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO(comboAlgorithm)->entry));
+      temp = (gchar *) gtk_entry_get_text (GTK_ENTRY (GTK_COMBO(comboAlgorithm)->entry));
       params->algo = gpa_algorithm_from_string (temp);
-      temp = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (comboKeysize)->entry));
+      temp = (gchar *) gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (comboKeysize)->entry));
       params->keysize = atoi (temp);
 
       params->generate_revocation \
