@@ -20,13 +20,17 @@
 
 #include <sys/stat.h>
 #include <gtk/gtk.h>
+#include <errno.h>
+#include <gpgme.h>
 #include "gpa.h"
 #include "gpapastrings.h"
 #include "icons.h"
 #include "gtktools.h"
 #include "gpawidgets.h"
 #include "gpawizard.h"
-
+#include "qdchkpwd.h"
+#include "gpgmetools.h"
+#include "keyexportdlg.h"
 
 /*
  * The key generation wizard
@@ -47,35 +51,7 @@
 static gchar *
 string_strip_dup (gchar * string)
 {
-  return g_strstrip (xstrdup (string));
-}
-
-/* Return TRUE if filename is a directory */
-gboolean
-isdir (const gchar * filename)
-{
-  struct stat statbuf;
-
-  return (stat (filename, &statbuf) == 0
-	  && S_ISDIR (statbuf.st_mode));
-}
-
-/* Return a copy of filename if it's a directory, its (malloced)
- * basename otherwise */
-gchar *
-file_dirname (const gchar * filename)
-{
-  gchar * result;
-  if (!isdir (filename))
-    {
-      result = g_dirname (filename);
-    }
-  else
-    {
-      result = xstrdup (filename);
-    }
-
-  return result;
+  return g_strstrip (g_strdup (string));
 }
 
 
@@ -100,11 +76,6 @@ typedef struct {
   GtkAccelGroup * accel_group;
   GdkPixmap * genkey_pixmap;
   GdkPixmap * backup_pixmap;
-  gboolean create_backups;
-  GpapaPublicKey * public_key;
-  GpapaSecretKey * secret_key;
-  gchar * pubkey_filename;
-  gchar * seckey_filename;
   gboolean successful;
 } GPAKeyGenWizard;
 
@@ -166,7 +137,7 @@ gpa_keygen_wizard_simple_get_text (GtkWidget * vbox)
   GtkWidget * entry;
 
   entry = gtk_object_get_data (GTK_OBJECT (vbox), "gpa_keygen_entry");
-  return string_strip_dup (gtk_entry_get_text (GTK_ENTRY (entry)));
+  return string_strip_dup ((gchar *) gtk_entry_get_text (GTK_ENTRY (entry)));
 }
 
 
@@ -197,7 +168,7 @@ gpa_keygen_wizard_name_validate (gpointer data)
       result = FALSE;
     }
 
-  free (name);
+  g_free (name);
   return result;
 }
 
@@ -208,12 +179,12 @@ gpa_keygen_wizard_email_page (GPAKeyGenWizard * keygen_wizard)
     (keygen_wizard,
      _("Please insert your email address."
        "\n\n"
-       " Your email adress will be part of the"
+       " Your email address will be part of the"
        " new key to make it easier for others to"
        " identify keys."
-       " If you have several email adresses, you can add further email"
+       " If you have several email addresses, you can add further email"
        " adresses later."),
-     _("Your Email Adress:"));
+     _("Your Email Address:"));
 }
 
 
@@ -235,7 +206,7 @@ gpa_keygen_wizard_email_validate (gpointer data)
   /* FIXME: we should do much more checking. Best would be exactly the
    * same checks gpg does in interactive mode with --gen-key */
 
-  free (email);
+  g_free (email);
   return result;
 }
 
@@ -248,7 +219,7 @@ gpa_keygen_wizard_comment_page (GPAKeyGenWizard * keygen_wizard)
      _("If you want you can supply a comment that further identifies"
        " the key to other users."
        " The comment is especially useful if you generate several keys"
-       " for the same email adress."
+       " for the same email address."
        " The comment is completely optional."
        " Leave it empty if you don't have a use for it."),
      _("Comment:"));
@@ -286,7 +257,7 @@ gpa_keygen_wizard_password_page (GPAKeyGenWizard * keygen_wizard)
   table = gtk_table_new (2, 2, FALSE);
   gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, TRUE, 5);
 
-  label = gtk_label_new (_("Password:"));
+  label = gtk_label_new (_("Passphrase: "));
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
 
@@ -299,7 +270,7 @@ gpa_keygen_wizard_password_page (GPAKeyGenWizard * keygen_wizard)
   gtk_object_set_data (GTK_OBJECT (vbox), "gpa_wizard_focus_child",
 		       (gpointer)entry);
 
-  label = gtk_label_new (_("Repeat Password:"));
+  label = gtk_label_new (_("Repeat Passphrase: "));
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
 
@@ -322,7 +293,7 @@ gpa_keygen_wizard_password_get_password (GtkWidget * vbox)
 {
   GtkWidget * entry = gtk_object_get_data (GTK_OBJECT (vbox),
 					   "gpa_keygen_passwd");
-  return gtk_entry_get_text (GTK_ENTRY (entry));
+  return (gchar *) gtk_entry_get_text (GTK_ENTRY (entry));
 }
 
 /* Validate the password in both entries and return TRUE if they're OK,
@@ -345,17 +316,41 @@ gpa_keygen_wizard_password_validate (gpointer data)
   if (strcmp (gtk_entry_get_text (GTK_ENTRY (entry_passwd)),
 	      gtk_entry_get_text (GTK_ENTRY (entry_repeat))) != 0)
     {
-      gpa_window_error (_("Please insert the same password in "
-			  "\"Password\" and \"Repeat Password\"."),
+      gpa_window_error (_("In \"Passphrase\" and \"Repeat passphrase\",\n"
+                          "you must enter the same passphrase."),
 			  keygen_wizard->window);
       result = FALSE;
     }
   else if (strlen (gtk_entry_get_text (GTK_ENTRY (entry_passwd))) == 0)
     {
-      gpa_window_error (_("Please insert a password\n"),
+      gpa_window_error (_("You did not enter a passphrase.\n"
+                          "It is needed to protect your private key."),
 			  keygen_wizard->window);
       result = FALSE;
     }
+  else if (strlen (gtk_entry_get_text (GTK_ENTRY (entry_passwd))) < 10
+           || qdchkpwd ((char*)gtk_entry_get_text (GTK_ENTRY (entry_passwd))) < 0.6)
+    {
+      GtkWidget *dialog;
+      
+      dialog = gtk_message_dialog_new (GTK_WINDOW (keygen_wizard->window),
+                                       GTK_DIALOG_MODAL,
+                                       GTK_MESSAGE_WARNING,
+                                       GTK_BUTTONS_NONE,
+                                       _("Warning: You have entered a "
+                                         "passphrase\n"
+                                         "that is obviously not secure.\n\n"
+                                         "Please enter a new passphrase."));
+      gtk_dialog_add_buttons (GTK_DIALOG (dialog), 
+                              _("_Enter new passphrase"), GTK_RESPONSE_CANCEL,
+                              _("Take this one _anyway"), GTK_RESPONSE_OK,
+                              NULL);
+      if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_CANCEL)
+        {
+          result = FALSE;
+        }
+      gtk_widget_destroy (dialog);
+    }
 
   return result;
 }
@@ -394,164 +389,6 @@ gpa_keygen_wizard_backup_page (GPAKeyGenWizard * keygen_wizard)
   return vbox;
 }
 
-
-static gboolean
-gpa_keygen_wizard_backup_action (gpointer data)
-{
-  GPAKeyGenWizard * keygen_wizard = data;
-  GtkWidget * radio;
-  gboolean do_backup;
-
-  radio = gtk_object_get_data (GTK_OBJECT (keygen_wizard->backup_page),
-			       "gpa_keygen_backup");
-  do_backup = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio));
-
-  keygen_wizard->create_backups = do_backup;
-
-  if (!do_backup)
-    {
-      gpa_wizard_next_page_no_action (keygen_wizard->wizard);
-      keygen_wizard->successful
-	= gpa_keygen_wizard_generate_action (keygen_wizard);
-    }
-  return TRUE;
-}
-
-
-static void
-gpa_keygen_wizard_backup_dir_browse (GtkWidget * button, gpointer param)
-{
-  GtkWidget * entry = param;
-  gchar * filename;
-  gchar * dirname;
-
-  filename = gpa_get_load_file_name (gtk_widget_get_toplevel (button),
-				     _("Backup Directory"), NULL);
-  if (filename)
-    {
-      dirname = file_dirname (filename);
-      gtk_entry_set_text (GTK_ENTRY (entry), dirname);
-      free (filename);
-      free (dirname);
-    }
-}
-  
-  
-
-static GtkWidget *
-gpa_keygen_wizard_backup_dir_page (GPAKeyGenWizard * keygen_wizard)
-{
-  GtkWidget * vbox;
-  GtkWidget * description;
-  GtkWidget * hbox;
-  GtkWidget * entry;
-  GtkWidget * label;
-  GtkWidget * button;
-  
-  vbox = gtk_vbox_new (FALSE, 0);
-
-  description = gtk_label_new (_("Please enter a directory where your"
-				 " backup keys should be saved.\n\n"
-				 "GPA will create two files in that directory:"
-				 " pub_key.asc and sec_key.asc"));
-  gtk_box_pack_start (GTK_BOX (vbox), description, TRUE, TRUE, 0);
-  gtk_misc_set_alignment (GTK_MISC (description), 0.0, 0.0);
-  gtk_label_set_line_wrap (GTK_LABEL (description), TRUE);
-  gtk_label_set_justify (GTK_LABEL (description), GTK_JUSTIFY_LEFT);
-
-  hbox = gtk_hbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 5);
-
-  label = gtk_label_new (_("Directory:"));
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-
-  entry = gtk_entry_new ();
-  gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 5);
-  gtk_signal_connect (GTK_OBJECT (entry), "activate",
-		      GTK_SIGNAL_FUNC (switch_to_next_page), keygen_wizard);
-
-  button = gpa_button_new (keygen_wizard->accel_group, _("B_rowse..."));
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      GTK_SIGNAL_FUNC (gpa_keygen_wizard_backup_dir_browse),
-		      entry);
-
-  gtk_object_set_data (GTK_OBJECT (vbox), "gpa_keygen_entry", entry);
-  gtk_object_set_data (GTK_OBJECT (vbox), "gpa_wizard_focus_child", entry);
-
-  return vbox;
-}
-
-static gchar *
-gpa_keygen_wizard_backup_get_text (GtkWidget * vbox)
-{
-  GtkWidget * entry;
-
-  entry = gtk_object_get_data (GTK_OBJECT (vbox), "gpa_keygen_entry");
-  return string_strip_dup (gtk_entry_get_text (GTK_ENTRY (entry)));
-}
-
-static gboolean
-gpa_keygen_wizard_backup_dir_action (gpointer data)
-{
-  GPAKeyGenWizard * keygen_wizard = data;
-  gchar * dir;
-  struct stat statbuf;
-  gboolean result = FALSE;
-  const gchar * buttons[] = {_("_Overwrite"), _("_Cancel"), NULL};
-  gchar * message;
-  gchar * reply;
-  
-  dir = gpa_keygen_wizard_backup_get_text (keygen_wizard->backup_dir_page);
-
-  if (isdir (dir))
-    {
-      /* FIXME: we should also test for permissions */
-      keygen_wizard->pubkey_filename = g_strconcat (dir, "/pub_key.asc", NULL);
-      keygen_wizard->seckey_filename = g_strconcat (dir, "/sec_key.asc", NULL);
-      result = TRUE;
-      if (stat (keygen_wizard->pubkey_filename, &statbuf) == 0)
-	{
-	  message = g_strdup_printf (_("The file %s already exists.\n"
-				       "Do you want to overwrite it?"),
-				     keygen_wizard->pubkey_filename);
-	  reply = gpa_message_box_run (keygen_wizard->window, _("File exists"),
-				       message, buttons);
-	  if (!reply || strcmp (reply, _("_Overwrite")) != 0)
-	    result = FALSE;
-	  free (message);
-	}
-
-      if (result && stat (keygen_wizard->pubkey_filename, &statbuf) == 0)
-	{
-	  message = g_strdup_printf (_("The file %s already exists.\n"
-				       "Do you want to overwrite it?"),
-				     keygen_wizard->seckey_filename);
-	  reply = gpa_message_box_run (keygen_wizard->window, _("File exists"),
-				       message, buttons);
-	  if (!reply || strcmp (reply, _("_Overwrite")) != 0)
-	    result = FALSE;
-	  free (message);
-	}
-    }
-  else
-    {
-      gpa_window_error (_("Please enter a valid directory"),
-			  keygen_wizard->window);
-    }
-
-  if (result)
-    {
-      keygen_wizard->successful
-	= gpa_keygen_wizard_generate_action (keygen_wizard);
-    }
-
-  free (dir);
-  return result;
-}
-
-
 static GtkWidget *
 gpa_keygen_wizard_message_page (const gchar * description_text)
 {
@@ -574,8 +411,8 @@ static GtkWidget *
 gpa_keygen_wizard_wait_page (void)
 {
   return gpa_keygen_wizard_message_page
-    (_("Your key is being generated\n\n"
-       "Even on fast computers this may take a while. Please be patient\n"));
+    (_("Your key is being generated.\n\n"
+       "Even on fast computers this may take a while. Please be patient."));
 }
 
 
@@ -589,50 +426,39 @@ gpa_keygen_wizard_final_page (void)
 }
 
 
-/* Generate a key pair for the given name, email, comment and password.
- * Algorithm and keysize are set to default values suitable for new
- * users. The parent parameter is used as the parent widget for error
- * message boxes */
-static gboolean
-gpa_keygen_generate_key(gchar * name, gchar * email, gchar * comment,
-			gchar * password, GPAKeyGenWizard * keygen_wizard)
-{
-  /* default values for newbie mode */
-  GpapaAlgo algo = GPAPA_ALGO_BOTH;
-  gint keysize = 1024;
-
-  gpapa_create_key_pair (&(keygen_wizard->public_key),
-			 &(keygen_wizard->secret_key), password,
-			 algo, keysize, name, email, comment,
-			 gpa_callback, keygen_wizard->window);
-  return global_lastCallbackResult == GPAPA_ACTION_FINISHED;
-}
-
-
-/* Extract the values the user entered and call gpa_keygen_generate_key.
+/* Extract the values the user entered and call gpa_generate_key.
  * Return TRUE if the key was created successfully.
  */
 static gboolean
 gpa_keygen_wizard_generate_action (gpointer data)
 {
-  GPAKeyGenWizard * keygen_wizard = data;
-  gchar * name;
-  gchar * email;
-  gchar * comment;
-  gchar * passwd;
-  gchar * backup_dir;
+  GPAKeyGenWizard *keygen_wizard = data;
+  GPAKeyGenParameters params;
+  GpgmeError err;
+  gchar *fpr;
+  gboolean do_backup;
+  GtkWidget *radio;
 
-  gboolean successful;
+  /* Shall we make backups? */
+  radio = gtk_object_get_data (GTK_OBJECT (keygen_wizard->backup_page),
+			       "gpa_keygen_backup");
+  do_backup = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio));
 
-  name = gpa_keygen_wizard_simple_get_text (keygen_wizard->name_page);
-  email = gpa_keygen_wizard_simple_get_text (keygen_wizard->email_page);
-  comment = gpa_keygen_wizard_simple_get_text (keygen_wizard->comment_page);
-  passwd = gpa_keygen_wizard_password_get_password(keygen_wizard->passwd_page);
-  if (keygen_wizard->create_backups)
-    backup_dir = gpa_keygen_wizard_backup_get_text (keygen_wizard
-						    ->backup_dir_page);
-  else
-    backup_dir = NULL;
+  /* The User ID */
+  params.userID = gpa_keygen_wizard_simple_get_text (keygen_wizard->name_page);
+  params.email = gpa_keygen_wizard_simple_get_text (keygen_wizard->email_page);
+  params.comment = gpa_keygen_wizard_simple_get_text (keygen_wizard
+                                                      ->comment_page);
+  params.password = gpa_keygen_wizard_password_get_password (keygen_wizard
+                                                             ->passwd_page);
+
+  /* default values for newbie mode */
+  params.algo = GPA_KEYGEN_ALGO_DSA_ELGAMAL;
+  params.keysize = 1024;
+  params.expiryDate = NULL;
+  params.interval = 0;
+  params.generate_revocation = FALSE;
+  params.send_to_server = FALSE;
 
   /* Switch to the next page showing the "wait" message. */
   gpa_wizard_next_page_no_action (keygen_wizard->wizard);
@@ -641,28 +467,22 @@ gpa_keygen_wizard_generate_action (gpointer data)
   while (gtk_events_pending())
     gtk_main_iteration();
 
-  successful = gpa_keygen_generate_key (name, email, comment, passwd,
-					keygen_wizard);
-
-  if (successful && keygen_wizard->create_backups)
+  err = gpa_generate_key (&params, &fpr);
+  if (err != GPGME_No_Error)
     {
-      gpapa_public_key_export (keygen_wizard->public_key,
-			       keygen_wizard->pubkey_filename,
-			       GPAPA_ARMOR, gpa_callback,
-			       keygen_wizard->window);
-      gpapa_secret_key_export (keygen_wizard->secret_key,
-			       keygen_wizard->seckey_filename,
-			       GPAPA_ARMOR, gpa_callback,
-			       keygen_wizard->window);
+      gpa_gpgme_error (err);
     }
-  
-  free (name);
-  free (email);
-  free (comment);
-  free (backup_dir);
-  
-  keygen_wizard->successful = successful;
-  return successful;
+  if (do_backup)
+    {
+      key_backup_dialog_run (keygen_wizard->window, fpr);
+    }
+  g_free (fpr);
+  g_free (params.userID);
+  g_free (params.email);
+  g_free (params.comment);
+
+  keygen_wizard->successful = TRUE;
+  return TRUE;
 }
 
 
@@ -690,11 +510,9 @@ free_keygen_wizard (gpointer data)
 {
   GPAKeyGenWizard * keygen_wizard = data;
 
-  free (keygen_wizard->pubkey_filename);
-  free (keygen_wizard->seckey_filename);
   gdk_pixmap_unref (keygen_wizard->genkey_pixmap);
   gdk_pixmap_unref (keygen_wizard->backup_pixmap);
-  free (keygen_wizard);
+  g_free (keygen_wizard);
 }
 
 
@@ -730,10 +548,8 @@ gpa_keygen_wizard_run (GtkWidget * parent)
 
 
 
-  keygen_wizard = xmalloc (sizeof (*keygen_wizard));
+  keygen_wizard = g_malloc (sizeof (*keygen_wizard));
   keygen_wizard->successful = FALSE;
-  keygen_wizard->pubkey_filename = NULL;
-  keygen_wizard->seckey_filename = NULL;
   keygen_wizard->genkey_pixmap = gpa_create_icon_pixmap (parent,
 							 "wizard_genkey",
 							 NULL);
@@ -744,7 +560,7 @@ gpa_keygen_wizard_run (GtkWidget * parent)
   accel_group = gtk_accel_group_new ();
   keygen_wizard->accel_group = accel_group;
 
-  window = gtk_window_new (GTK_WINDOW_DIALOG);
+  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   keygen_wizard->window = window;
   gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
   gtk_window_set_title (GTK_WINDOW (window), _("Generate key"));
@@ -762,8 +578,9 @@ gpa_keygen_wizard_run (GtkWidget * parent)
   keygen_wizard->pixmap_widget = pixmap_widget;
   gtk_box_pack_start (GTK_BOX (hbox), pixmap_widget, FALSE, TRUE, 0);
   
-  wizard = gpa_wizard_new (accel_group, gpa_keygen_wizard_close,
-			   keygen_wizard);
+  wizard = gpa_wizard_new (accel_group,
+                           (GtkSignalFunc) gpa_keygen_wizard_close,
+			   (GtkSignalFunc) keygen_wizard);
   keygen_wizard->wizard = wizard;
   gtk_box_pack_start (GTK_BOX (hbox), wizard, TRUE, TRUE, 0);
   gpa_wizard_set_page_switched (wizard, page_switched, keygen_wizard);
@@ -790,13 +607,8 @@ gpa_keygen_wizard_run (GtkWidget * parent)
   keygen_wizard->backup_page = gpa_keygen_wizard_backup_page (keygen_wizard);
   gpa_wizard_append_page (wizard, keygen_wizard->backup_page,
 			  NULL, NULL,
-			  gpa_keygen_wizard_backup_action, keygen_wizard);
+			  gpa_keygen_wizard_generate_action, keygen_wizard);
 
-  keygen_wizard->backup_dir_page
-    = gpa_keygen_wizard_backup_dir_page (keygen_wizard);
-  gpa_wizard_append_page (wizard, keygen_wizard->backup_dir_page,
-			  NULL, _("F_inish"),
-			  gpa_keygen_wizard_backup_dir_action, keygen_wizard);
   /* Don't use F as the accelerator in "Finish" because Meta-F is
    * already bound in the entry widget */
 
